#include <FL/fl_ask.H>

#include "ImpressionistModel.h"
#include "ImpressionistView.h"
#include "DotBrush.h"

//==========[ ImpressionistModel Methods ]=====================================

//----------[ Default Constructor ]--------------------------------------------

ImpressionistModel::ImpressionistModel() {
	mBitmapWidth = -1;
	mOriginalBitmap = NULL;
	mPaintedBitmap = NULL;

	mBrushSize = 10;
}

//----------[ loadImage ]------------------------------------------------------

int ImpressionistModel::loadImage(char *iname) {
	unsigned char* bitmapData;

	if ( (bitmapData=readBMP(iname, mBitmapWidth, mBitmapHeight)) == NULL ) {
		fl_alert( "Unable to load bitmap file" );
		return 0;
	}

	// release any previously stored bitmap data
	if ( mOriginalBitmap ) delete [] mOriginalBitmap;
	if ( mPaintedBitmap ) delete [] mPaintedBitmap;

	mOriginalBitmap = bitmapData;

	// allocate space for draw view
	mPaintedBitmap = new unsigned char[mBitmapWidth*mBitmapHeight*3];
	memset( mPaintedBitmap, 0, mBitmapWidth*mBitmapHeight*3 );

	return 1;
}

//----------[ saveImage ]------------------------------------------------------

int ImpressionistModel::saveImage( char *fname ) const {
	writeBMP( fname, mBitmapWidth, mBitmapHeight, mPaintedBitmap );
	return 1;
}

//----------[ getOriginalPixel ]-----------------------------------------------

unsigned char* ImpressionistModel::getOriginalPixel( int x, int y ) {
	if( x < 0 )
		x = 0;
	else if( x >= mBitmapWidth )
		x = mBitmapWidth-1;

	if( y < 0 )
		y = 0;
	else if( y >= mBitmapHeight )
		y = mBitmapHeight-1;

	return (unsigned char*)(mOriginalBitmap + 3 * (y*mBitmapWidth + x));
}

unsigned char* ImpressionistModel::getOriginalPixel( const Point p ) {
	return getOriginalPixel( p.x, p.y );
}
